@@ -266,11 +266,9 @@ func getRemoteRefs(l *tasklog.Logger) (map[string][]*git.Ref, error) {
 	}
 
 	if !migrateSkipFetch {
-		w := l.Waiter(fmt.Sprintf("migrate: %s", tr.Tr.Get("Fetching remote refs")))
-		if err := git.Fetch(remotes...); err != nil {
+		if err := fetchRemoteRefs(l, remotes); err != nil {
 			return nil, err
 		}
-		w.Complete()
 	}
 
 	for _, remote := range remotes {
@@ -278,7 +276,7 @@ func getRemoteRefs(l *tasklog.Logger) (map[string][]*git.Ref, error) {
 		if migrateSkipFetch {
 			refsForRemote, err = git.CachedRemoteRefs(remote)
 		} else {
-			refsForRemote, err = git.RemoteRefs(remote)
+			refsForRemote, err = git.RemoteRefs(remote, true)
 		}
 
 		if err != nil {
@@ -291,6 +289,13 @@ func getRemoteRefs(l *tasklog.Logger) (map[string][]*git.Ref, error) {
 	return refs, nil
 }
 
+func fetchRemoteRefs(l *tasklog.Logger, remotes []string) error {
+	w := l.Waiter(fmt.Sprintf("migrate: %s", tr.Tr.Get("Fetching remote refs")))
+	defer w.Complete()
+
+	return git.Fetch(remotes...)
+}
+
 // formatRefName returns the fully-qualified name for the given Git reference
 // "ref".
 func formatRefName(ref *git.Ref, remote string) string {
