@@ -1,6 +1,11 @@
 package fs
 
-import "testing"
+import (
+	"os"
+	"testing"
+
+	"github.com/stretchr/testify/assert"
+)
 
 func TestDecodeNone(t *testing.T) {
 	evaluate(t, "A:\\some\\regular\\windows\\path", "A:\\some\\regular\\windows\\path")
@@ -24,3 +29,16 @@ func evaluate(t *testing.T, input string, expected string) {
 	}
 
 }
+
+func TestRepositoryPermissions(t *testing.T) {
+	m := map[os.FileMode]os.FileMode{
+		0777: 0666,
+		0755: 0644,
+		0700: 0600,
+	}
+	for k, v := range m {
+		fs := Filesystem{repoPerms: v}
+		assert.Equal(t, k, fs.RepositoryPermissions(true))
+		assert.Equal(t, v, fs.RepositoryPermissions(false))
+	}
+}
