@@ -4,15 +4,14 @@ import (
 	"bytes"
 	"encoding/hex"
 	"io"
-	"io/ioutil"
-	"path/filepath"
 	"reflect"
 	"strconv"
 	"strings"
 	"testing"
 
-	"github.com/git-lfs/git-lfs/filepathfilter"
-	"github.com/git-lfs/git-lfs/git/odb"
+	"github.com/git-lfs/git-lfs/v3/errors"
+	"github.com/git-lfs/git-lfs/v3/filepathfilter"
+	"github.com/git-lfs/gitobj/v2"
 	"github.com/stretchr/testify/assert"
 )
 
@@ -21,8 +20,8 @@ func TestRewriterRewritesHistory(t *testing.T) {
 	r := NewRewriter(db)
 
 	tip, err := r.Rewrite(&RewriteOptions{Include: []string{"refs/heads/master"},
-		BlobFn: func(path string, b *odb.Blob) (*odb.Blob, error) {
-			contents, err := ioutil.ReadAll(b.Contents)
+		BlobFn: func(path string, b *gitobj.Blob) (*gitobj.Blob, error) {
+			contents, err := io.ReadAll(b.Contents)
 			if err != nil {
 				return nil, err
 			}
@@ -34,7 +33,7 @@ func TestRewriterRewritesHistory(t *testing.T) {
 
 			rewritten := strconv.Itoa(n + 1)
 
-			return &odb.Blob{
+			return &gitobj.Blob{
 				Contents: strings.NewReader(rewritten),
 				Size:     int64(len(rewritten)),
 			}, nil
@@ -82,8 +81,8 @@ func TestRewriterRewritesOctopusMerges(t *testing.T) {
 	r := NewRewriter(db)
 
 	tip, err := r.Rewrite(&RewriteOptions{Include: []string{"refs/heads/master"},
-		BlobFn: func(path string, b *odb.Blob) (*odb.Blob, error) {
-			return &odb.Blob{
+		BlobFn: func(path string, b *gitobj.Blob) (*gitobj.Blob, error) {
+			return &gitobj.Blob{
 				Contents: io.MultiReader(b.Contents, strings.NewReader("_new")),
 				Size:     b.Size + int64(len("_new")),
 			}, nil
@@ -129,15 +128,15 @@ func TestRewriterVisitsPackedObjects(t *testing.T) {
 	var contents []byte
 
 	_, err := r.Rewrite(&RewriteOptions{Include: []string{"refs/heads/master"},
-		BlobFn: func(path string, b *odb.Blob) (*odb.Blob, error) {
+		BlobFn: func(path string, b *gitobj.Blob) (*gitobj.Blob, error) {
 			var err error
 
-			contents, err = ioutil.ReadAll(b.Contents)
+			contents, err = io.ReadAll(b.Contents)
 			if err != nil {
 				return nil, err
 			}
 
-			return &odb.Blob{
+			return &gitobj.Blob{
 				Contents: bytes.NewReader(contents),
 				Size:     int64(len(contents)),
 			}, nil
@@ -155,7 +154,7 @@ func TestRewriterDoesntVisitUnchangedSubtrees(t *testing.T) {
 	seen := make(map[string]int)
 
 	_, err := r.Rewrite(&RewriteOptions{Include: []string{"refs/heads/master"},
-		BlobFn: func(path string, b *odb.Blob) (*odb.Blob, error) {
+		BlobFn: func(path string, b *gitobj.Blob) (*gitobj.Blob, error) {
 			seen[path] = seen[path] + 1
 
 			return b, nil
@@ -165,7 +164,7 @@ func TestRewriterDoesntVisitUnchangedSubtrees(t *testing.T) {
 	assert.Nil(t, err)
 
 	assert.Equal(t, 2, seen["a.txt"])
-	assert.Equal(t, 1, seen[filepath.Join("subdir", "b.txt")])
+	assert.Equal(t, 1, seen["subdir/b.txt"])
 }
 
 func TestRewriterVisitsUniqueEntriesWithIdenticalContents(t *testing.T) {
@@ -173,12 +172,12 @@ func TestRewriterVisitsUniqueEntriesWithIdenticalContents(t *testing.T) {
 	r := NewRewriter(db)
 
 	tip, err := r.Rewrite(&RewriteOptions{Include: []string{"refs/heads/master"},
-		BlobFn: func(path string, b *odb.Blob) (*odb.Blob, error) {
+		BlobFn: func(path string, b *gitobj.Blob) (*gitobj.Blob, error) {
 			if path == "b.txt" {
 				return b, nil
 			}
 
-			return &odb.Blob{
+			return &gitobj.Blob{
 				Contents: strings.NewReader("changed"),
 				Size:     int64(len("changed")),
 			}, nil
@@ -205,7 +204,7 @@ func TestRewriterIgnoresPathsThatDontMatchFilter(t *testing.T) {
 	include := []string{"*.txt"}
 	exclude := []string{"subdir/*.txt"}
 
-	filter := filepathfilter.New(include, exclude)
+	filter := filepathfilter.New(include, exclude, filepathfilter.GitIgnore)
 
 	db := DatabaseFromFixture(t, "non-repeated-subtrees.git")
 	r := NewRewriter(db, WithFilter(filter))
@@ -213,7 +212,7 @@ func TestRewriterIgnoresPathsThatDontMatchFilter(t *testing.T) {
 	seen := make(map[string]int)
 
 	_, err := r.Rewrite(&RewriteOptions{Include: []string{"refs/heads/master"},
-		BlobFn: func(path string, b *odb.Blob) (*odb.Blob, error) {
+		BlobFn: func(path string, b *gitobj.Blob) (*gitobj.Blob, error) {
 			seen[path] = seen[path] + 1
 
 			return b, nil
@@ -222,27 +221,27 @@ func TestRewriterIgnoresPathsThatDontMatchFilter(t *testing.T) {
 
 	assert.Nil(t, err)
 	assert.Equal(t, 1, seen["a.txt"])
-	assert.Equal(t, 0, seen[filepath.Join("subdir", "b.txt")])
+	assert.Equal(t, 0, seen["subdir/b.txt"])
 }
 
 func TestRewriterAllowsAdditionalTreeEntries(t *testing.T) {
 	db := DatabaseFromFixture(t, "linear-history.git")
 	r := NewRewriter(db)
 
-	extra, err := db.WriteBlob(&odb.Blob{
+	extra, err := db.WriteBlob(&gitobj.Blob{
 		Contents: strings.NewReader("extra\n"),
 		Size:     int64(len("extra\n")),
 	})
 	assert.Nil(t, err)
 
 	tip, err := r.Rewrite(&RewriteOptions{Include: []string{"refs/heads/master"},
-		BlobFn: func(path string, b *odb.Blob) (*odb.Blob, error) {
+		BlobFn: func(path string, b *gitobj.Blob) (*gitobj.Blob, error) {
 			return b, nil
 		},
 
-		TreeCallbackFn: func(path string, tr *odb.Tree) (*odb.Tree, error) {
-			return &odb.Tree{
-				Entries: append(tr.Entries, &odb.TreeEntry{
+		TreeCallbackFn: func(path string, tr *gitobj.Tree) (*gitobj.Tree, error) {
+			return &gitobj.Tree{
+				Entries: append(tr.Entries, &gitobj.TreeEntry{
 					Name:     "extra.txt",
 					Filemode: 0100644,
 					Oid:      extra,
@@ -293,6 +292,109 @@ func TestRewriterAllowsAdditionalTreeEntries(t *testing.T) {
 	AssertBlobContents(t, db, tree3, "extra.txt", "extra\n")
 }
 
+// CallbackCall is a structure recording information pertinent to when a
+// *githistory.Rewrite called either BlobFn, TreePreCallbackFn, or
+// TreeCallbackFn.
+type CallbackCall struct {
+	Type string
+	Path string
+}
+
+var (
+	// collectCalls is a function that returns a *RewriteOptions that
+	// updates a pointer to a slice of `*CallbackCall`'s with each call that
+	// is received.
+	collectCalls = func(calls *[]*CallbackCall) *RewriteOptions {
+		return &RewriteOptions{Include: []string{"refs/heads/master"},
+			BlobFn: func(path string, b *gitobj.Blob) (*gitobj.Blob, error) {
+				*calls = append(*calls, &CallbackCall{
+					Type: "blob",
+					Path: path,
+				})
+				return b, nil
+			},
+
+			TreePreCallbackFn: func(path string, t *gitobj.Tree) error {
+				*calls = append(*calls, &CallbackCall{
+					Type: "tree-pre",
+					Path: path,
+				})
+				return nil
+			},
+
+			TreeCallbackFn: func(path string, t *gitobj.Tree) (*gitobj.Tree, error) {
+				*calls = append(*calls, &CallbackCall{
+					Type: "tree-post",
+					Path: path,
+				})
+				return t, nil
+			},
+		}
+	}
+)
+
+func TestHistoryRewriterCallbacks(t *testing.T) {
+	var calls []*CallbackCall
+
+	db := DatabaseFromFixture(t, "linear-history.git")
+	r := NewRewriter(db)
+
+	_, err := r.Rewrite(collectCalls(&calls))
+
+	assert.Nil(t, err)
+
+	assert.Len(t, calls, 9)
+	assert.Equal(t, calls[0], &CallbackCall{Type: "tree-pre", Path: "/"})
+	assert.Equal(t, calls[1], &CallbackCall{Type: "blob", Path: "hello.txt"})
+	assert.Equal(t, calls[2], &CallbackCall{Type: "tree-post", Path: "/"})
+	assert.Equal(t, calls[3], &CallbackCall{Type: "tree-pre", Path: "/"})
+	assert.Equal(t, calls[4], &CallbackCall{Type: "blob", Path: "hello.txt"})
+	assert.Equal(t, calls[5], &CallbackCall{Type: "tree-post", Path: "/"})
+	assert.Equal(t, calls[6], &CallbackCall{Type: "tree-pre", Path: "/"})
+	assert.Equal(t, calls[7], &CallbackCall{Type: "blob", Path: "hello.txt"})
+	assert.Equal(t, calls[8], &CallbackCall{Type: "tree-post", Path: "/"})
+}
+
+func TestHistoryRewriterCallbacksSubtrees(t *testing.T) {
+	var calls []*CallbackCall
+
+	db := DatabaseFromFixture(t, "non-repeated-subtrees.git")
+	r := NewRewriter(db)
+
+	_, err := r.Rewrite(collectCalls(&calls))
+
+	assert.Nil(t, err)
+
+	assert.Len(t, calls, 8)
+	assert.Equal(t, calls[0], &CallbackCall{Type: "tree-pre", Path: "/"})
+	assert.Equal(t, calls[1], &CallbackCall{Type: "blob", Path: "a.txt"})
+	assert.Equal(t, calls[2], &CallbackCall{Type: "tree-post", Path: "/"})
+	assert.Equal(t, calls[3], &CallbackCall{Type: "tree-pre", Path: "/"})
+	assert.Equal(t, calls[4], &CallbackCall{Type: "tree-pre", Path: "/subdir"})
+	assert.Equal(t, calls[5], &CallbackCall{Type: "blob", Path: "subdir/b.txt"})
+	assert.Equal(t, calls[6], &CallbackCall{Type: "tree-post", Path: "/subdir"})
+	assert.Equal(t, calls[7], &CallbackCall{Type: "tree-post", Path: "/"})
+}
+
+func TestHistoryRewriterTreePreCallbackPropagatesErrors(t *testing.T) {
+	expected := errors.Errorf("my error")
+
+	db := DatabaseFromFixture(t, "linear-history.git")
+	r := NewRewriter(db)
+
+	_, err := r.Rewrite(&RewriteOptions{Include: []string{"refs/heads/master"},
+		BlobFn: func(path string, b *gitobj.Blob) (*gitobj.Blob, error) {
+			return b, nil
+		},
+
+		TreePreCallbackFn: func(path string, t *gitobj.Tree) error {
+			return expected
+		},
+	})
+
+	assert.Equal(t, err, expected)
+}
+
 func TestHistoryRewriterUseOriginalParentsForPartialMigration(t *testing.T) {
 	db := DatabaseFromFixture(t, "linear-history-with-tags.git")
 	r := NewRewriter(db)
@@ -301,7 +403,7 @@ func TestHistoryRewriterUseOriginalParentsForPartialMigration(t *testing.T) {
 		Include: []string{"refs/heads/master"},
 		Exclude: []string{"refs/tags/middle"},
 
-		BlobFn: func(path string, b *odb.Blob) (*odb.Blob, error) {
+		BlobFn: func(path string, b *gitobj.Blob) (*gitobj.Blob, error) {
 			return b, nil
 		},
 	})
@@ -338,10 +440,10 @@ func TestHistoryRewriterUpdatesRefs(t *testing.T) {
 
 		UpdateRefs: true,
 
-		BlobFn: func(path string, b *odb.Blob) (*odb.Blob, error) {
+		BlobFn: func(path string, b *gitobj.Blob) (*gitobj.Blob, error) {
 			suffix := strings.NewReader("_suffix")
 
-			return &odb.Blob{
+			return &gitobj.Blob{
 				Contents: io.MultiReader(b.Contents, suffix),
 				Size:     b.Size + int64(suffix.Len()),
 			}, nil
@@ -361,7 +463,7 @@ func TestHistoryRewriterUpdatesRefs(t *testing.T) {
 }
 
 func TestHistoryRewriterReturnsFilter(t *testing.T) {
-	f := filepathfilter.New([]string{"a"}, []string{"b"})
+	f := filepathfilter.New([]string{"a"}, []string{"b"}, filepathfilter.GitIgnore)
 	r := NewRewriter(nil, WithFilter(f))
 
 	expected := reflect.ValueOf(f).Elem().Addr().Pointer()
@@ -370,3 +472,18 @@ func TestHistoryRewriterReturnsFilter(t *testing.T) {
 	assert.Equal(t, expected, got,
 		"git/githistory: expected Rewriter.Filter() to return same *filepathfilter.Filter instance")
 }
+
+// debug is meant to be called from a defer statement to aide in debugging a
+// test failure among any in this file.
+//
+// Callers are expected to call it immediately after calling the Rewrite()
+// function.
+func debug(t *testing.T, db *gitobj.ObjectDatabase, tip []byte, err error) {
+	root, ok := db.Root()
+
+	t.Log(strings.Repeat("*", 80))
+	t.Logf("* root=%s, ok=%t\n", root, ok)
+	t.Logf("* tip=%x\n", tip)
+	t.Logf("* err=%s\n", err)
+	t.Log(strings.Repeat("*", 80))
+}
