@@ -5,12 +5,12 @@ import (
 	"encoding/hex"
 	"fmt"
 	"io"
-	"io/ioutil"
 	"regexp"
 	"strings"
 	"sync"
 
-	"github.com/git-lfs/git-lfs/errors"
+	"github.com/git-lfs/git-lfs/v3/errors"
+	"github.com/git-lfs/git-lfs/v3/tr"
 	"github.com/rubyist/tracerx"
 )
 
@@ -23,9 +23,9 @@ const (
 	ScanRefsMode ScanningMode = iota
 	// ScanAllMode will scan all history.
 	ScanAllMode
-	// ScanLeftToRemoteMode will scan the difference between any included
+	// ScanRangeToRemoteMode will scan the difference between any included
 	// SHA1s and a remote tracking ref.
-	ScanLeftToRemoteMode
+	ScanRangeToRemoteMode
 )
 
 // RevListOrder is a constant type that allows for variation in the ordering of
@@ -65,7 +65,7 @@ func (o RevListOrder) Flag() (string, bool) {
 	case TopoRevListOrder:
 		return "--topo-order", true
 	default:
-		panic(fmt.Sprintf("git/rev_list_scanner: unknown RevListOrder %d", o))
+		panic(fmt.Sprintf("git/rev_list_scanner: %s", tr.Tr.Get("unknown RevListOrder %d", o)))
 	}
 }
 
@@ -76,7 +76,7 @@ type ScanRefsOptions struct {
 	// Mode is the scan mode to apply, see above.
 	Mode ScanningMode
 	// Remote is the current remote to scan against, if using
-	// ScanLeftToRemoveMode.
+	// ScanRangeToRemoteMode.
 	Remote string
 	// SkipDeletedBlobs specifies whether or not to traverse into commit
 	// ancestry (revealing potentially deleted (unreferenced) blobs, trees,
@@ -153,10 +153,6 @@ var (
 	// ambiguousRegex is a regular expression matching the output of stderr
 	// when ambiguous refnames are encountered.
 	ambiguousRegex = regexp.MustCompile(`warning: refname (.*) is ambiguous`)
-
-	// z40 is a regular expression matching the empty blob/commit/tree
-	// SHA: "0000000000000000000000000000000000000000".
-	z40 = regexp.MustCompile(`\^?0{40}`)
 )
 
 // NewRevListScanner instantiates a new RevListScanner instance scanning all
@@ -172,7 +168,10 @@ func NewRevListScanner(include, excluded []string, opt *ScanRefsOptions) (*RevLi
 		return nil, err
 	}
 
-	cmd := gitNoLFS(args...).Cmd
+	cmd, err := gitNoLFS(args...)
+	if err != nil {
+		return nil, err
+	}
 	if len(opt.WorkingDir) > 0 {
 		cmd.Dir = opt.WorkingDir
 	}
@@ -195,13 +194,13 @@ func NewRevListScanner(include, excluded []string, opt *ScanRefsOptions) (*RevLi
 	return &RevListScanner{
 		s: bufio.NewScanner(stdout),
 		closeFn: func() error {
-			msg, _ := ioutil.ReadAll(stderr)
+			msg, _ := io.ReadAll(stderr)
 
 			// First check if there was a non-zero exit code given
 			// when Wait()-ing on the command execution.
 			if err := cmd.Wait(); err != nil {
-				return errors.Errorf("Error in git %s: %v %s",
-					strings.Join(args, " "), err, msg)
+				return errors.New(tr.Tr.Get("Error in `git %s`: %v %s",
+					strings.Join(args, " "), err, msg))
 			}
 
 			// If the command exited cleanly, but found an ambiguous
@@ -210,7 +209,7 @@ func NewRevListScanner(include, excluded []string, opt *ScanRefsOptions) (*RevLi
 			// `git-rev-list(1)` does not treat ambiguous refnames
 			// as fatal (non-zero exit status), but we do.
 			if am := ambiguousRegex.FindSubmatch(msg); len(am) > 1 {
-				return errors.Errorf("ref %s is ambiguous", am[1])
+				return errors.New(tr.Tr.Get("ref %q is ambiguous", am[1]))
 			}
 			return nil
 		},
@@ -246,23 +245,25 @@ func revListArgs(include, exclude []string, opt *ScanRefsOptions) (io.Reader, []
 			args = append(args, "--do-walk")
 		}
 
-		args = append(args, includeExcludeShas(include, exclude)...)
+		stdin = strings.NewReader(strings.Join(
+			includeExcludeShas(include, exclude), "\n"))
 	case ScanAllMode:
 		args = append(args, "--all")
-	case ScanLeftToRemoteMode:
+	case ScanRangeToRemoteMode:
+		args = append(args, "--ignore-missing")
 		if len(opt.SkippedRefs) == 0 {
-			args = append(args, includeExcludeShas(include, exclude)...)
 			args = append(args, "--not", "--remotes="+opt.Remote)
+			stdin = strings.NewReader(strings.Join(
+				includeExcludeShas(include, exclude), "\n"))
 		} else {
-			args = append(args, "--stdin")
 			stdin = strings.NewReader(strings.Join(
 				append(includeExcludeShas(include, exclude), opt.SkippedRefs...), "\n"),
 			)
 		}
 	default:
-		return nil, nil, errors.Errorf("unknown scan type: %d", opt.Mode)
+		return nil, nil, errors.New(tr.Tr.Get("unknown scan type: %d", opt.Mode))
 	}
-	return stdin, append(args, "--"), nil
+	return stdin, append(args, "--stdin", "--"), nil
 }
 
 func includeExcludeShas(include, exclude []string) []string {
@@ -286,13 +287,15 @@ func nonZeroShas(all []string) []string {
 	nz := make([]string, 0, len(all))
 
 	for _, sha := range all {
-		if len(sha) > 0 && !z40.MatchString(sha) {
+		if len(sha) > 0 && !IsZeroObjectID(sha) {
 			nz = append(nz, sha)
 		}
 	}
 	return nz
 }
 
+var startsWithObjectID = regexp.MustCompile(fmt.Sprintf(`\A%s`, ObjectIDRegex))
+
 // Name is an optional field that gives the name of the object (if the object is
 // a tree, blob).
 //
@@ -343,19 +346,23 @@ func (s *RevListScanner) scan() ([]byte, string, error) {
 	}
 
 	line := strings.TrimSpace(s.s.Text())
-	if len(line) < 40 {
+	if len(line) < ObjectIDLengths[0] {
 		return nil, "", nil
 	}
 
-	sha1, err := hex.DecodeString(line[:40])
+	oidhex := startsWithObjectID.FindString(line)
+	if len(oidhex) == 0 {
+		return nil, "", errors.New(tr.Tr.Get("missing OID in line (got %q)", line))
+	}
+	oid, err := hex.DecodeString(oidhex)
 	if err != nil {
 		return nil, "", err
 	}
 
 	var name string
-	if len(line) > 40 {
-		name = line[41:]
+	if len(line) > len(oidhex) {
+		name = line[len(oidhex)+1:]
 	}
 
-	return sha1, name, nil
+	return oid, name, nil
 }
