@@ -4,6 +4,16 @@
 
 ensure_git_version_isnt $VERSION_LOWER "2.2.0"
 
+# Check for a libnss3 dependency in Git until we drop support for CentOS 7.
+GIT_LIBNSS=0
+if [ "$IS_WINDOWS" -eq 0 -a "$IS_MAC" -eq 0 ]; then
+  GIT_LIBNSS="$(ldd "$(git --exec-path)"/git-remote-https | grep -c '^\s*libnss3\.' || true)"
+fi
+export GIT_LIBNSS
+
+export CREDSDIR="$REMOTEDIR/creds-clone"
+setup_creds
+
 begin_test "clone"
 (
   set -e
@@ -14,6 +24,8 @@ begin_test "clone"
 
   git lfs track "*.dat" 2>&1 | tee track.log
   grep "Tracking \"\*.dat\"" track.log
+  git add .gitattributes
+  git commit -m "Track *.dat"
 
   # generate some test data & commits with random LFS data
   echo "[
@@ -48,8 +60,8 @@ begin_test "clone"
   grep "Cloning into" lfsclone.log
   grep "Downloading LFS objects:" lfsclone.log
   # should be no filter errors
-  [ ! $(grep "filter" lfsclone.log) ]
-  [ ! $(grep "error" lfsclone.log) ]
+  grep "filter" lfsclone.log && exit 1
+  grep "error" lfsclone.log && exit 1
   # should be cloned into location as per arg
   [ -d "$newclonedir" ]
 
@@ -69,8 +81,8 @@ begin_test "clone"
   grep "Cloning into" lfsclone.log
   grep "Downloading LFS objects:" lfsclone.log
   # should be no filter errors
-  [ ! $(grep "filter" lfsclone.log) ]
-  [ ! $(grep "error" lfsclone.log) ]
+  grep "filter" lfsclone.log && exit 1
+  grep "error" lfsclone.log && exit 1
   # clone location should be implied
   [ -d "$reponame" ]
 
@@ -83,23 +95,49 @@ begin_test "clone"
     assert_clean_status
   popd
 
+  # Now check clone with standard 'git clone' and smudge download
+  rm -rf "$reponame"
+  git clone "$GITSERVER/$reponame" 2>&1 | tee clone.log
+  if [ "0" -ne "${PIPESTATUS[0]}" ]; then
+    echo >&2 "fatal: expected clone to succeed ..."
+    exit 1
+  fi
+  grep "Cloning into" clone.log
+  [ -d "$reponame" ]
+
+  pushd "$reponame"
+    [ $(wc -c < "file1.dat") -eq 110 ]
+    [ $(wc -c < "file2.dat") -eq 75 ]
+    [ $(wc -c < "file3.dat") -eq 66 ]
+    assert_hooks "$(dot_git_dir)"
+    [ ! -e "lfs" ]
+    [ "6" -eq "$(find "$(dot_git_dir)/lfs/objects" -type f | wc -l)" ]
+    assert_clean_status
+  popd
 )
 end_test
 
 begin_test "cloneSSL"
 (
   set -e
-  if $TRAVIS; then
-    echo "Skipping SSL tests, Travis has weird behaviour in validating custom certs, test locally only"
+
+  if [ "$GIT_LIBNSS" -eq 1 ]; then
+    echo "skip: libnss does not support the Go httptest server certificate"
     exit 0
   fi
 
+  if [ "$IS_WINDOWS" -eq 1 ]; then
+    git config --global "http.sslBackend" "openssl"
+  fi
+
   reponame="test-cloneSSL"
   setup_remote_repo "$reponame"
   clone_repo_ssl "$reponame" "$reponame"
 
   git lfs track "*.dat" 2>&1 | tee track.log
   grep "Tracking \"\*.dat\"" track.log
+  git add .gitattributes
+  git commit -m "Track *.dat"
 
   # generate some test data & commits with random LFS data
   echo "[
@@ -123,48 +161,100 @@ begin_test "cloneSSL"
 
   newclonedir="testcloneSSL1"
   git lfs clone "$SSLGITSERVER/$reponame" "$newclonedir" 2>&1 | tee lfsclone.log
-  assert_clean_status
   grep "Cloning into" lfsclone.log
-  grep "Git LFS:" lfsclone.log
+  grep "Downloading LFS objects:" lfsclone.log
   # should be no filter errors
-  [ ! $(grep "filter" lfsclone.log) ]
-  [ ! $(grep "error" lfsclone.log) ]
+  grep "filter" lfsclone.log && exit 1
+  grep "error" lfsclone.log && exit 1
   # should be cloned into location as per arg
   [ -d "$newclonedir" ]
 
   # check a few file sizes to make sure pulled
   pushd "$newclonedir"
-  [ $(wc -c < "file1.dat") -eq 100 ]
-  [ $(wc -c < "file2.dat") -eq 75 ]
-  [ $(wc -c < "file3.dat") -eq 30 ]
-  assert_hooks "$(dot_git_dir)"
+    [ $(wc -c < "file1.dat") -eq 100 ]
+    [ $(wc -c < "file2.dat") -eq 75 ]
+    [ $(wc -c < "file3.dat") -eq 30 ]
+    assert_hooks "$(dot_git_dir)"
+    [ ! -e "lfs" ]
+    assert_clean_status
   popd
 
-
   # Now check SSL clone with standard 'git clone' and smudge download
   rm -rf "$reponame"
-  git clone "$SSLGITSERVER/$reponame"
+  git clone "$SSLGITSERVER/$reponame" 2>&1 | tee clone.log
+  if [ "0" -ne "${PIPESTATUS[0]}" ]; then
+    echo >&2 "fatal: expected clone to succeed ..."
+    exit 1
+  fi
+  grep "Cloning into" clone.log
+  [ -d "$reponame" ]
+
+  pushd "$reponame"
+    [ $(wc -c < "file1.dat") -eq 100 ]
+    [ $(wc -c < "file2.dat") -eq 75 ]
+    [ $(wc -c < "file3.dat") -eq 30 ]
+    assert_hooks "$(dot_git_dir)"
+    [ ! -e "lfs" ]
+    [ "3" -eq "$(find "$(dot_git_dir)/lfs/objects" -type f | wc -l)" ]
+    assert_clean_status
+  popd
 )
 end_test
 
 begin_test "clone ClientCert"
 (
   set -e
-  if $TRAVIS; then
-    echo "Skipping SSL tests, Travis has weird behaviour in validating custom certs, test locally only"
+
+  if [ "$GIT_LIBNSS" -eq 1 ]; then
+    echo "skip: libnss does not support the Go httptest server certificate"
     exit 0
   fi
 
+  if [ "$IS_WINDOWS" -eq 1 ]; then
+    git config --global "http.sslBackend" "openssl"
+  fi
+
+  # Note that the record files we create in the $CREDSDIR directory are not
+  # used until we set the "http.sslCertPasswordProtected" option to "true"
+  # and the "http.<url>.sslKey" option with the path to our TLS/SSL client
+  # certificate's encrypted private key file.  (The PEM certificate file
+  # itself is not encrypted and does not contain the private key.)
+  #
+  # When these options are set, however, Git and Git LFS will independently
+  # invoke "git credential fill" to retrieve the passphrase for the
+  # encrypted private key.  Because the "http.sslCertPasswordProtected"
+  # option is set, Git will query the credential helper, passing a
+  # "protocol=cert" line and a "path=<certfile>" line with the path
+  # from the "http.<url>.sslCert" option.  Note that this path refers
+  # to our unencrypted certificate file; Git does not use the path to
+  # the encrypted private key file from the "http.<url>.sslKey" option
+  # in its query to the credential helper.
+  #
+  # Separately, the Git LFS client will detect that the private key file
+  # specified by the "http.<url>.sslKey" option is encrypted, and so will
+  # invoke "git credential fill" to retrieve its passphrase, passing a
+  # "protocol=cert" line and a "path=<keyfile>" line with the path
+  # from the "http.<url>.sslKey" option.
+  #
+  # In order to satisfy both requests, our git-credential-lfstest helper
+  # therefore needs two record files, both with the passphrase for the
+  # encrypted private key file.  For Git, one is associated with the path
+  # to the certificate file, and for Git LFS, one is associated with the
+  # path to the key file.
+  write_creds_file "::pass" "$CREDSDIR/--$(echo "$LFS_CLIENT_CERT_FILE" | tr / -)"
+  write_creds_file "::pass" "$CREDSDIR/--$(echo "$LFS_CLIENT_KEY_FILE_ENCRYPTED" | tr / -)"
+
+  git config --global "http.$LFS_CLIENT_CERT_URL/.sslCert" "$LFS_CLIENT_CERT_FILE"
+  git config --global "http.$LFS_CLIENT_CERT_URL/.sslKey" "$LFS_CLIENT_KEY_FILE"
+
   reponame="test-cloneClientCert"
   setup_remote_repo "$reponame"
   clone_repo_clientcert "$reponame" "$reponame"
-  if [ $(grep -c "client-cert-mac-openssl" clone_client_cert.log) -gt 0 ]; then
-    echo "Skipping due to SSL client cert bug in Git"
-    exit 0
-  fi
 
   git lfs track "*.dat" 2>&1 | tee track.log
   grep "Tracking \"\*.dat\"" track.log
+  git add .gitattributes
+  git commit -m "Track *.dat"
 
   # generate some test data & commits with random LFS data
   echo "[
@@ -184,20 +274,22 @@ begin_test "clone ClientCert"
   git push origin main
 
   # Now clone again with 'git lfs clone', test specific clone dir
+  # Test with both unencrypted and encrypted client certificate keys
   cd "$TRASHDIR"
 
-  for i in "$LFS_CLIENT_KEY_FILE" "$LFS_CLIENT_KEY_FILE_ENCRYPTED"
-  do
-    export GIT_SSL_CERT_PASSWORD_PROTECTED=1
-    git config --global http.$LFS_CLIENT_CERT_URL/.sslKey "$i"
+  for enc in "false" "true"; do
+    if [ "$enc" = "true" ]; then
+      git config --global "http.$LFS_CLIENT_CERT_URL/.sslKey" "$LFS_CLIENT_KEY_FILE_ENCRYPTED"
+      git config --global "http.sslCertPasswordProtected" "$enc"
+    fi
 
-    newclonedir="testcloneClietCert1"
-    rm -fr "$newclonedir"
+    newclonedir="${reponame}-${enc}"
     git lfs clone "$CLIENTCERTGITSERVER/$reponame" "$newclonedir" 2>&1 | tee lfsclone.log
     grep "Cloning into" lfsclone.log
+    grep "Downloading LFS objects:" lfsclone.log
     # should be no filter errors
-    [ ! $(grep "filter" lfsclone.log) ]
-    [ ! $(grep "error" lfsclone.log) ]
+    grep "filter" lfsclone.log && exit 1
+    grep "error" lfsclone.log && exit 1
     # should be cloned into location as per arg
     [ -d "$newclonedir" ]
 
@@ -207,16 +299,30 @@ begin_test "clone ClientCert"
       [ $(wc -c < "file2.dat") -eq 75 ]
       [ $(wc -c < "file3.dat") -eq 30 ]
       assert_hooks "$(dot_git_dir)"
+      [ ! -e "lfs" ]
+      assert_clean_status
+    popd
+
+    # Now check clone with standard 'git clone' and smudge download
+    rm -rf "$reponame"
+    git clone "$CLIENTCERTGITSERVER/$reponame" 2>&1 | tee clone.log
+    if [ "0" -ne "${PIPESTATUS[0]}" ]; then
+      echo >&2 "fatal: expected clone to succeed ..."
+      exit 1
+    fi
+    grep "Cloning into" clone.log
+    [ -d "$reponame" ]
+
+    pushd "$reponame"
+      [ $(wc -c < "file1.dat") -eq 100 ]
+      [ $(wc -c < "file2.dat") -eq 75 ]
+      [ $(wc -c < "file3.dat") -eq 30 ]
+      assert_hooks "$(dot_git_dir)"
+      [ ! -e "lfs" ]
+      [ "3" -eq "$(find "$(dot_git_dir)/lfs/objects" -type f | wc -l)" ]
       assert_clean_status
     popd
   done
-
-  git config --global http.$LFS_CLIENT_CERT_URL/.sslKey "$LFS_CLIENT_KEY_FILE"
-
-  # Now check SSL clone with standard 'git clone' and smudge download
-  rm -rf "$reponame"
-  git clone "$CLIENTCERTGITSERVER/$reponame"
-
 )
 end_test
 
@@ -224,28 +330,70 @@ begin_test "clone ClientCert with homedir certs"
 (
   set -e
 
-  # Windows triggers a credential helper problem.
-  if [ $IS_WINDOWS -eq 1 ]; then
+  if [ "$GIT_LIBNSS" -eq 1 ]; then
+    echo "skip: libnss does not support the Go httptest server certificate"
     exit 0
   fi
 
+  if [ "$IS_WINDOWS" -eq 1 ]; then
+    git config --global "http.sslBackend" "openssl"
+  fi
+
+  cp "$LFS_CLIENT_CERT_FILE" "$HOME/lfs-client-cert-file"
+  cp "$LFS_CLIENT_KEY_FILE" "$HOME/lfs-client-key-file"
+  cp "$LFS_CLIENT_KEY_FILE_ENCRYPTED" "$HOME/lfs-client-key-file-encrypted"
+
+  # Note that the record files we create in the $CREDSDIR directory are not
+  # used until we set the "http.sslCertPasswordProtected" option to "true"
+  # and the "http.<url>.sslKey" option with the path to our TLS/SSL client
+  # certificate's encrypted private key file.  (The PEM certificate file
+  # itself is not encrypted and does not contain the private key.)
+  #
+  # When these options are set, however, Git and Git LFS will independently
+  # invoke "git credential fill" to retrieve the passphrase for the
+  # encrypted private key.  Because the "http.sslCertPasswordProtected"
+  # option is set, Git will query the credential helper, passing a
+  # "protocol=cert" line and a "path=<certfile>" line with the path
+  # from the "http.<url>.sslCert" option.  Note that this path refers
+  # to our unencrypted certificate file; Git does not use the path to
+  # the encrypted private key file from the "http.<url>.sslKey" option
+  # in its query to the credential helper.
+  #
+  # Separately, the Git LFS client will detect that the private key file
+  # specified by the "http.<url>.sslKey" option is encrypted, and so will
+  # invoke "git credential fill" to retrieve its passphrase, passing a
+  # "protocol=cert" line and a "path=<keyfile>" line with the path
+  # from the "http.<url>.sslKey" option.
+  #
+  # In order to satisfy both requests, our git-credential-lfstest helper
+  # therefore needs two record files, both with the passphrase for the
+  # encrypted private key file.  For Git, one is associated with the path
+  # to the certificate file, and for Git LFS, one is associated with the
+  # path to the key file.
+  if [ "$IS_WINDOWS" -eq 1 ]; then
+    # In our MSYS2 CI environment we have to convert the Unix-style path
+    # in $HOME, which starts with /tmp/, into a path of the form /a/...
+    # so that the credential record filename we create from it matches
+    # the one our git-credential-lfstest helper will construct from the
+    # "path" values it receives from Git and Git LFS.
+    homedir="$(cygpath -m "$HOME" | sed 's,^\([A-Z]\):,/\L\1,')"
+  else
+    homedir="$HOME"
+  fi
+  write_creds_file "::pass" "$CREDSDIR/--$(echo "$homedir/lfs-client-cert-file" | tr / -)"
+  write_creds_file "::pass" "$CREDSDIR/--$(echo "$homedir/lfs-client-key-file-encrypted" | tr / -)"
+
+  git config --global "http.$LFS_CLIENT_CERT_URL/.sslCert" "~/lfs-client-cert-file"
+  git config --global "http.$LFS_CLIENT_CERT_URL/.sslKey" "~/lfs-client-key-file"
+
   reponame="test-cloneClientCert-homedir"
-
-  cp "$LFS_CLIENT_KEY_FILE" "$HOME/lfs-client-key-file"
-  cp "$LFS_CLIENT_CERT_FILE" "$HOME/lfs-client-cert-file"
-
-  git config --global http.$LFS_CLIENT_CERT_URL/.sslKey "~/lfs-client-key-file"
-  git config --global http.$LFS_CLIENT_CERT_URL/.sslCert "~/lfs-client-cert-file"
-
   setup_remote_repo "$reponame"
   clone_repo_clientcert "$reponame" "$reponame"
-  if [ $(grep -c "client-cert-mac-openssl" clone_client_cert.log) -gt 0 ]; then
-    echo "Skipping due to SSL client cert bug in Git"
-    exit 0
-  fi
 
   git lfs track "*.dat" 2>&1 | tee track.log
   grep "Tracking \"\*.dat\"" track.log
+  git add .gitattributes
+  git commit -m "Track *.dat"
 
   # generate some test data & commits with random LFS data
   echo "[
@@ -263,6 +411,57 @@ begin_test "clone ClientCert with homedir certs"
   ]" | lfstest-testutils addcommits
 
   git push origin main
+
+  # Now clone again with 'git lfs clone', test specific clone dir
+  # Test with both unencrypted and encrypted client certificate keys
+  cd "$TRASHDIR"
+
+  for enc in "false" "true"; do
+    if [ "$enc" = "true" ]; then
+      git config --global "http.$LFS_CLIENT_CERT_URL/.sslKey" "~/lfs-client-key-file-encrypted"
+      git config --global "http.sslCertPasswordProtected" "$enc"
+    fi
+
+    newclonedir="${reponame}-${enc}"
+    git lfs clone "$CLIENTCERTGITSERVER/$reponame" "$newclonedir" 2>&1 | tee lfsclone.log
+    grep "Cloning into" lfsclone.log
+    grep "Downloading LFS objects:" lfsclone.log
+    # should be no filter errors
+    grep "filter" lfsclone.log && exit 1
+    grep "error" lfsclone.log && exit 1
+    # should be cloned into location as per arg
+    [ -d "$newclonedir" ]
+
+    # check a few file sizes to make sure pulled
+    pushd "$newclonedir"
+      [ $(wc -c < "file1.dat") -eq 100 ]
+      [ $(wc -c < "file2.dat") -eq 75 ]
+      [ $(wc -c < "file3.dat") -eq 30 ]
+      assert_hooks "$(dot_git_dir)"
+      [ ! -e "lfs" ]
+      assert_clean_status
+    popd
+
+    # Now check clone with standard 'git clone' and smudge download
+    rm -rf "$reponame"
+    git clone "$CLIENTCERTGITSERVER/$reponame" 2>&1 | tee clone.log
+    if [ "0" -ne "${PIPESTATUS[0]}" ]; then
+      echo >&2 "fatal: expected clone to succeed ..."
+      exit 1
+    fi
+    grep "Cloning into" clone.log
+    [ -d "$reponame" ]
+
+    pushd "$reponame"
+      [ $(wc -c < "file1.dat") -eq 100 ]
+      [ $(wc -c < "file2.dat") -eq 75 ]
+      [ $(wc -c < "file3.dat") -eq 30 ]
+      assert_hooks "$(dot_git_dir)"
+      [ ! -e "lfs" ]
+      [ "3" -eq "$(find "$(dot_git_dir)/lfs/objects" -type f | wc -l)" ]
+      assert_clean_status
+    popd
+  done
 )
 end_test
 
@@ -270,15 +469,14 @@ begin_test "clone with flags"
 (
   set -e
 
-  git config --global http.$LFS_CLIENT_CERT_URL/.sslCert "$LFS_CLIENT_CERT_FILE"
-  git config --global http.$LFS_CLIENT_CERT_URL/.sslKey "$LFS_CLIENT_KEY_FILE"
-
   reponame="$(basename "$0" ".sh")-flags"
   setup_remote_repo "$reponame"
   clone_repo "$reponame" "$reponame"
 
   git lfs track "*.dat" 2>&1 | tee track.log
   grep "Tracking \"\*.dat\"" track.log
+  git add .gitattributes
+  git commit -m "Track *.dat"
 
   # generate some test data & commits with random LFS data
   echo "[
@@ -324,11 +522,13 @@ begin_test "clone with flags"
   # specific test for --branch and --origin
   git lfs clone --branch branch2 --recurse-submodules --origin differentorigin "$GITSERVER/$reponame" "$newclonedir"
   pushd "$newclonedir"
-  # this file is only on branch2
-  [ -e "fileonbranch2.dat" ]
-  # confirm remote is called differentorigin
-  git remote get-url differentorigin
-  assert_hooks "$(dot_git_dir)"
+    # this file is only on branch2
+    [ -e "fileonbranch2.dat" ]
+    # confirm remote is called differentorigin
+    git remote get-url differentorigin
+    assert_hooks "$(dot_git_dir)"
+    [ ! -e "lfs" ]
+    assert_clean_status
   popd
   rm -rf "$newclonedir"
 
@@ -342,6 +542,10 @@ begin_test "clone with flags"
   [ -e "$newclonedir/.git" ]
   [ -d "$gitdir/objects" ]
   assert_hooks "$gitdir"
+  pushd "$newclonedir"
+    [ ! -e "lfs" ]
+    assert_clean_status
+  popd
   rm -rf "$newclonedir"
   rm -rf "$gitdir"
 
@@ -396,23 +600,27 @@ begin_test "clone (with include/exclude args)"
   local_reponame="clone_with_includes"
   git lfs clone "$GITSERVER/$reponame" "$local_reponame" -I "a*.dat"
   pushd "$local_reponame"
-  assert_local_object "$contents_a_oid" 1
-  refute_local_object "$contents_b_oid"
-  [ "a" = "$(cat a.dat)" ]
-  [ "a" = "$(cat a-dupe.dat)" ]
-  [ "$(pointer $contents_a_oid 1)" = "$(cat dupe-a.dat)" ]
-  [ "$(pointer $contents_b_oid 1)" = "$(cat b.dat)" ]
-  assert_hooks "$(dot_git_dir)"
+    assert_local_object "$contents_a_oid" 1
+    refute_local_object "$contents_b_oid"
+    [ "a" = "$(cat a.dat)" ]
+    [ "a" = "$(cat a-dupe.dat)" ]
+    [ "$(pointer $contents_a_oid 1)" = "$(cat dupe-a.dat)" ]
+    [ "$(pointer $contents_b_oid 1)" = "$(cat b.dat)" ]
+    assert_hooks "$(dot_git_dir)"
+    [ ! -e "lfs" ]
+    assert_clean_status
   popd
 
   local_reponame="clone_with_excludes"
   git lfs clone "$GITSERVER/$reponame" "$local_reponame" -I "b.dat" -X "a.dat"
   pushd "$local_reponame"
-  assert_local_object "$contents_b_oid" 1
-  refute_local_object "$contents_a_oid"
-  [ "$(pointer $contents_a_oid 1)" = "$(cat a.dat)" ]
-  [ "b" = "$(cat b.dat)" ]
-  assert_hooks "$(dot_git_dir)"
+    assert_local_object "$contents_b_oid" 1
+    refute_local_object "$contents_a_oid"
+    [ "$(pointer $contents_a_oid 1)" = "$(cat a.dat)" ]
+    [ "b" = "$(cat b.dat)" ]
+    assert_hooks "$(dot_git_dir)"
+    [ ! -e "lfs" ]
+    assert_clean_status
   popd
 )
 end_test
@@ -457,34 +665,38 @@ begin_test "clone (with .lfsconfig)"
 
   pushd "$TRASHDIR"
 
-  echo "test: clone with lfs.fetchinclude in .lfsconfig"
-  local_reponame="clone_with_config_include"
-  set +x
-  git lfs clone "$GITSERVER/$reponame" "$local_reponame"
-  ok="$?"
-  set -x
-  if [ "0" -ne "$ok" ]; then
-    # TEMP: used to catch transient failure from above `clone` command, as in:
-    # https://github.com/git-lfs/git-lfs/pull/1782#issuecomment-267678319
-    echo >&2 "[!] \`git lfs clone $GITSERVER/$reponame $local_reponame\` failed"
-    git lfs logs last
+    echo "test: clone with lfs.fetchinclude in .lfsconfig"
+    local_reponame="clone_with_config_include"
+    set +x
+    git lfs clone "$GITSERVER/$reponame" "$local_reponame"
+    ok="$?"
+    set -x
+    if [ "0" -ne "$ok" ]; then
+      # TEMP: used to catch transient failure from above `clone` command, as in:
+      # https://github.com/git-lfs/git-lfs/pull/1782#issuecomment-267678319
+      echo >&2 "[!] \`git lfs clone $GITSERVER/$reponame $local_reponame\` failed"
+      git lfs logs last
 
-    exit 1
-  fi
-  pushd "$local_reponame"
-  assert_local_object "$contents_a_oid" 1
-  refute_local_object "$contents_b_oid"
-  assert_hooks "$(dot_git_dir)"
-  popd
+      exit 1
+    fi
+    pushd "$local_reponame"
+      assert_local_object "$contents_a_oid" 1
+      refute_local_object "$contents_b_oid"
+      assert_hooks "$(dot_git_dir)"
+      [ ! -e "lfs" ]
+      assert_clean_status
+    popd
 
-  echo "test: clone with lfs.fetchinclude in .lfsconfig, and args"
-  local_reponame="clone_with_config_include_and_args"
-  git lfs clone "$GITSERVER/$reponame" "$local_reponame" -I "b.dat"
-  pushd "$local_reponame"
-  refute_local_object "$contents_a_oid"
-  assert_local_object "$contents_b_oid" 1
-  assert_hooks "$(dot_git_dir)"
-  popd
+    echo "test: clone with lfs.fetchinclude in .lfsconfig, and args"
+    local_reponame="clone_with_config_include_and_args"
+    git lfs clone "$GITSERVER/$reponame" "$local_reponame" -I "b.dat"
+    pushd "$local_reponame"
+      refute_local_object "$contents_a_oid"
+      assert_local_object "$contents_b_oid" 1
+      assert_hooks "$(dot_git_dir)"
+      [ ! -e "lfs" ]
+      assert_clean_status
+    popd
 
   popd
 
@@ -499,24 +711,28 @@ begin_test "clone (with .lfsconfig)"
 
   pushd "$TRASHDIR"
 
-  echo "test: clone with lfs.fetchexclude in .lfsconfig"
-  local_reponame="clone_with_config_exclude"
-  git lfs clone "$GITSERVER/$reponame" "$local_reponame"
-  pushd "$local_reponame"
-  cat ".lfsconfig"
-  assert_local_object "$contents_b_oid" 1
-  refute_local_object "$contents_a_oid"
-  assert_hooks "$(dot_git_dir)"
-  popd
+    echo "test: clone with lfs.fetchexclude in .lfsconfig"
+    local_reponame="clone_with_config_exclude"
+    git lfs clone "$GITSERVER/$reponame" "$local_reponame"
+    pushd "$local_reponame"
+      cat ".lfsconfig"
+      assert_local_object "$contents_b_oid" 1
+      refute_local_object "$contents_a_oid"
+      assert_hooks "$(dot_git_dir)"
+      [ ! -e "lfs" ]
+      assert_clean_status
+    popd
 
-  echo "test: clone with lfs.fetchexclude in .lfsconfig, and args"
-  local_reponame="clone_with_config_exclude_and_args"
-  git lfs clone "$GITSERVER/$reponame" "$local_reponame" -I "a.dat" -X "b.dat"
-  pushd "$local_reponame"
-  assert_local_object "$contents_a_oid" 1
-  refute_local_object "$contents_b_oid"
-  assert_hooks "$(dot_git_dir)"
-  popd
+    echo "test: clone with lfs.fetchexclude in .lfsconfig, and args"
+    local_reponame="clone_with_config_exclude_and_args"
+    git lfs clone "$GITSERVER/$reponame" "$local_reponame" -I "a.dat" -X "b.dat"
+    pushd "$local_reponame"
+      assert_local_object "$contents_a_oid" 1
+      refute_local_object "$contents_b_oid"
+      assert_hooks "$(dot_git_dir)"
+      [ ! -e "lfs" ]
+      assert_clean_status
+    popd
 
   popd
 )
@@ -623,22 +839,24 @@ begin_test "clone with submodules"
 
   pushd "$TRASHDIR"
 
-  local_reponame="submod-clone"
-  git lfs clone --recursive "$GITSERVER/$reponame" "$local_reponame"
+    local_reponame="submod-clone"
+    git lfs clone --recursive "$GITSERVER/$reponame" "$local_reponame"
 
-  # check everything is where it should be
-  cd $local_reponame
-  assert_hooks "$(dot_git_dir)"
-  # check LFS store and working copy
-  assert_local_object "$contents_root_oid" "${#contents_root}"
-  [ $(wc -c < "root.dat") -eq ${#contents_root} ]
-  # and so on for nested subs
-  cd sub1
-  assert_local_object "$contents_sub1_oid" "${#contents_sub1}"
-  [ $(wc -c < "sub1.dat") -eq ${#contents_sub1} ]
-  cd sub2
-  assert_local_object "$contents_sub2_oid" "${#contents_sub2}"
-  [ $(wc -c < "sub2.dat") -eq ${#contents_sub2} ]
+    # check everything is where it should be
+    cd $local_reponame
+    assert_hooks "$(dot_git_dir)"
+    [ ! -e "lfs" ]
+    assert_clean_status
+    # check LFS store and working copy
+    assert_local_object "$contents_root_oid" "${#contents_root}"
+    [ $(wc -c < "root.dat") -eq ${#contents_root} ]
+    # and so on for nested subs
+    cd sub1
+    assert_local_object "$contents_sub1_oid" "${#contents_sub1}"
+    [ $(wc -c < "sub1.dat") -eq ${#contents_sub1} ]
+    cd sub2
+    assert_local_object "$contents_sub2_oid" "${#contents_sub2}"
+    [ $(wc -c < "sub2.dat") -eq ${#contents_sub2} ]
 
   popd
 )
@@ -679,7 +897,8 @@ begin_test "clone in current directory"
 
     assert_local_object "$contents_oid" 8
     assert_hooks "$(dot_git_dir)"
-    [ ! -f ./lfs ]
+    [ ! -e "lfs" ]
+    assert_clean_status
   popd
 )
 end_test
@@ -732,6 +951,8 @@ begin_test "clone (HTTP server/proxy require cookies)"
 
   git lfs track "*.dat" 2>&1 | tee track.log
   grep "Tracking \"\*.dat\"" track.log
+  git add .gitattributes
+  git commit -m "Track *.dat"
 
   # generate some test data & commits with random LFS data
   echo "[
@@ -766,8 +987,8 @@ begin_test "clone (HTTP server/proxy require cookies)"
   grep "Cloning into" lfsclone.log
   grep "Downloading LFS objects:" lfsclone.log
   # should be no filter errors
-  [ ! $(grep "filter" lfsclone.log) ]
-  [ ! $(grep "error" lfsclone.log) ]
+  grep "filter" lfsclone.log && exit 1
+  grep "error" lfsclone.log && exit 1
   # should be cloned into location as per arg
   [ -d "$newclonedir" ]
 
@@ -787,8 +1008,8 @@ begin_test "clone (HTTP server/proxy require cookies)"
   grep "Cloning into" lfsclone.log
   grep "Downloading LFS objects:" lfsclone.log
   # should be no filter errors
-  [ ! $(grep "filter" lfsclone.log) ]
-  [ ! $(grep "error" lfsclone.log) ]
+  grep "filter" lfsclone.log && exit 1
+  grep "error" lfsclone.log && exit 1
   # clone location should be implied
   [ -d "$reponame" ]
 
@@ -800,8 +1021,5 @@ begin_test "clone (HTTP server/proxy require cookies)"
     [ ! -e "lfs" ]
     assert_clean_status
   popd
-
-  # to avoid breaking t-credentials.sh
-  rm "$CREDSDIR/localhost"
 )
 end_test
