@@ -95,7 +95,7 @@ begin_test "fetch (shared repository)"
   rm -rf .git/lfs/objects
 
   git lfs fetch 2>&1 | tee fetch.log
-  ! grep "Could not scan" fetch.log
+  grep "Could not scan" fetch.log && exit 1
   assert_local_object "$contents_oid" 1
 
   git lfs fsck 2>&1 | tee fsck.log
@@ -298,6 +298,20 @@ begin_test "fetch with missing object"
 )
 end_test
 
+begin_test "fetch does not crash on empty key files"
+(
+  set -e
+  cd clone
+  rm -rf .git/lfs/objects
+
+  git config --local http.sslKey /dev/null
+  git config --local http.sslCert /dev/null
+
+  git lfs fetch origin main 2>&1 | tee fetch.log
+  grep "Error decoding PEM block" fetch.log
+)
+end_test
+
 begin_test "fetch-all"
 (
   set -e
@@ -628,6 +642,16 @@ begin_test "fetch raw remote url"
 )
 end_test
 
+begin_test "fetch with invalid ref"
+(
+  set -e
+  cd repo
+
+  git lfs fetch origin jibberish >fetch.log 2>&1 && exit 1
+  grep "Invalid ref argument" fetch.log
+)
+end_test
+
 begin_test "fetch with invalid remote"
 (
   set -e
