@@ -11,10 +11,12 @@ import (
 	"strings"
 	"time"
 
-	"github.com/git-lfs/git-lfs/errors"
-	"github.com/git-lfs/git-lfs/tools"
+	"github.com/git-lfs/git-lfs/v3/errors"
+	"github.com/git-lfs/git-lfs/v3/fs"
+	"github.com/git-lfs/git-lfs/v3/tools"
+	"github.com/git-lfs/git-lfs/v3/tr"
 
-	"github.com/git-lfs/git-lfs/subprocess"
+	"github.com/git-lfs/git-lfs/v3/subprocess"
 	"github.com/rubyist/tracerx"
 )
 
@@ -118,24 +120,22 @@ func (a *customAdapter) Begin(cfg AdapterConfig, cb ProgressCallback) error {
 	return a.adapterBase.Begin(&customAdapterConfig{AdapterConfig: cfg}, cb)
 }
 
-func (a *customAdapter) ClearTempStorage() error {
-	// no action requred
-	return nil
-}
-
 func (a *customAdapter) WorkerStarting(workerNum int) (interface{}, error) {
-
 	// Start a process per worker
 	// If concurrent = false we have already dialled back workers to 1
 	a.Trace("xfer: starting up custom transfer process %q for worker %d", a.name, workerNum)
-	cmd := subprocess.ExecCommand(a.path, a.args)
+	cmdName, cmdArgs := subprocess.FormatForShell(subprocess.ShellQuoteSingle(a.path), a.args)
+	cmd, err := subprocess.ExecCommand(cmdName, cmdArgs...)
+	if err != nil {
+		return nil, errors.New(tr.Tr.Get("failed to find custom transfer command %q remote: %v", a.path, err))
+	}
 	outp, err := cmd.StdoutPipe()
 	if err != nil {
-		return nil, fmt.Errorf("Failed to get stdout for custom transfer command %q remote: %v", a.path, err)
+		return nil, errors.New(tr.Tr.Get("failed to get stdout for custom transfer command %q remote: %v", a.path, err))
 	}
 	inp, err := cmd.StdinPipe()
 	if err != nil {
-		return nil, fmt.Errorf("Failed to get stdin for custom transfer command %q remote: %v", a.path, err)
+		return nil, errors.New(tr.Tr.Get("failed to get stdin for custom transfer command %q remote: %v", a.path, err))
 	}
 	// Capture stderr to trace
 	tracer := &traceWriter{}
@@ -143,7 +143,7 @@ func (a *customAdapter) WorkerStarting(workerNum int) (interface{}, error) {
 	cmd.Stderr = tracer
 	err = cmd.Start()
 	if err != nil {
-		return nil, fmt.Errorf("Failed to start custom transfer command %q remote: %v", a.path, err)
+		return nil, errors.New(tr.Tr.Get("failed to start custom transfer command %q remote: %v", a.path, err))
 	}
 	// Set up buffered reader/writer since we operate on lines
 	ctx := &customAdapterWorkerContext{workerNum, cmd, outp, bufio.NewReader(outp), inp, tracer}
@@ -159,7 +159,7 @@ func (a *customAdapter) WorkerStarting(workerNum int) (interface{}, error) {
 	}
 	if resp.Error != nil {
 		a.abortWorkerProcess(ctx)
-		return nil, fmt.Errorf("Error initializing custom adapter %q worker %d: %v", a.name, workerNum, resp.Error)
+		return nil, errors.New(tr.Tr.Get("error initializing custom adapter %q worker %d: %v", a.name, workerNum, resp.Error))
 	}
 
 	a.Trace("xfer: started custom adapter process %q for worker %d OK", a.path, workerNum)
@@ -202,7 +202,6 @@ func (a *customAdapter) readResponse(ctx *customAdapterWorkerContext) (*customAd
 // exchangeMessage sends a message to a process and reads a response if resp != nil
 // Only fatal errors to communicate return an error, errors may be embedded in reply
 func (a *customAdapter) exchangeMessage(ctx *customAdapterWorkerContext, req interface{}) (*customAdapterResponseMessage, error) {
-
 	err := a.sendMessage(ctx, req)
 	if err != nil {
 		return nil, err
@@ -232,7 +231,7 @@ func (a *customAdapter) shutdownWorkerProcess(ctx *customAdapterWorkerContext) e
 	case err := <-finishChan:
 		return err
 	case <-time.After(30 * time.Second):
-		return fmt.Errorf("Timeout while shutting down worker process %d", ctx.workerNum)
+		return errors.New(tr.Tr.Get("timeout while shutting down worker process %d", ctx.workerNum))
 	}
 }
 
@@ -259,12 +258,12 @@ func (a *customAdapter) WorkerEnding(workerNum int, ctx interface{}) {
 
 func (a *customAdapter) DoTransfer(ctx interface{}, t *Transfer, cb ProgressCallback, authOkFunc func()) error {
 	if ctx == nil {
-		return fmt.Errorf("Custom transfer %q was not properly initialized, see previous errors", a.name)
+		return errors.New(tr.Tr.Get("custom transfer %q was not properly initialized, see previous errors", a.name))
 	}
 
 	customCtx, ok := ctx.(*customAdapterWorkerContext)
 	if !ok {
-		return fmt.Errorf("Context object for custom transfer %q was of the wrong type", a.name)
+		return errors.New(tr.Tr.Get("context object for custom transfer %q was of the wrong type", a.name))
 	}
 	var authCalled bool
 
@@ -273,7 +272,7 @@ func (a *customAdapter) DoTransfer(ctx interface{}, t *Transfer, cb ProgressCall
 		return err
 	}
 	if rel == nil && !a.standalone {
-		return errors.Errorf("Object %s not found on the server.", t.Oid)
+		return errors.Errorf(tr.Tr.Get("Object %s not found on the server.", t.Oid))
 	}
 	var req *customAdapterTransferRequest
 	if a.direction == Upload {
@@ -297,7 +296,7 @@ func (a *customAdapter) DoTransfer(ctx interface{}, t *Transfer, cb ProgressCall
 		case "progress":
 			// Progress
 			if resp.Oid != t.Oid {
-				return fmt.Errorf("Unexpected oid %q in response, expecting %q", resp.Oid, t.Oid)
+				return errors.New(tr.Tr.Get("unexpected OID %q in response, expecting %q", resp.Oid, t.Oid))
 			}
 			if cb != nil {
 				cb(t.Name, t.Size, resp.BytesSoFar, resp.BytesSinceLast)
@@ -306,19 +305,19 @@ func (a *customAdapter) DoTransfer(ctx interface{}, t *Transfer, cb ProgressCall
 		case "complete":
 			// Download/Upload complete
 			if resp.Oid != t.Oid {
-				return fmt.Errorf("Unexpected oid %q in response, expecting %q", resp.Oid, t.Oid)
+				return errors.New(tr.Tr.Get("unexpected OID %q in response, expecting %q", resp.Oid, t.Oid))
 			}
 			if resp.Error != nil {
-				return fmt.Errorf("Error transferring %q: %v", t.Oid, resp.Error)
+				return errors.New(tr.Tr.Get("error transferring %q: %v", t.Oid, resp.Error))
 			}
 			if a.direction == Download {
 				// So we don't have to blindly trust external providers, check SHA
 				if err = tools.VerifyFileHash(t.Oid, resp.Path); err != nil {
-					return fmt.Errorf("Downloaded file failed checks: %v", err)
+					return errors.New(tr.Tr.Get("downloaded file failed checks: %v", err))
 				}
 				// Move file to final location
 				if err = tools.RenameFileCopyPermissions(resp.Path, t.Path); err != nil {
-					return fmt.Errorf("Failed to copy downloaded file: %v", err)
+					return errors.New(tr.Tr.Get("failed to copy downloaded file: %v", err))
 				}
 			} else if a.direction == Upload {
 				if err = verifyUpload(a.apiClient, a.remote, t); err != nil {
@@ -328,7 +327,7 @@ func (a *customAdapter) DoTransfer(ctx interface{}, t *Transfer, cb ProgressCall
 			wasAuthOk = true
 			complete = true
 		default:
-			return fmt.Errorf("Invalid message %q from custom adapter %q", resp.Event, a.name)
+			return errors.New(tr.Tr.Get("invalid message %q from custom adapter %q", resp.Event, a.name))
 		}
 		// Fall through from both progress and completion messages
 		// Call auth on first progress or success to free up other workers
@@ -341,15 +340,30 @@ func (a *customAdapter) DoTransfer(ctx interface{}, t *Transfer, cb ProgressCall
 	return nil
 }
 
-func newCustomAdapter(name string, dir Direction, path, args string, concurrent, standalone bool) *customAdapter {
-	c := &customAdapter{newAdapterBase(name, dir, nil), path, args, concurrent, 3, standalone}
+func newCustomAdapter(f *fs.Filesystem, name string, dir Direction, path, args string, concurrent, standalone bool) *customAdapter {
+	c := &customAdapter{newAdapterBase(f, name, dir, nil), path, args, concurrent, 3, standalone}
 	// self implements impl
 	c.transferImpl = c
 	return c
 }
 
+const (
+	standaloneFileName = "lfs-standalone-file"
+)
+
+func configureDefaultCustomAdapters(git Env, m *concreteManifest) {
+	newfunc := func(name string, dir Direction) Adapter {
+		standalone := m.standaloneTransferAgent != ""
+		return newCustomAdapter(m.fs, standaloneFileName, dir, "git-lfs", "standalone-file", false, standalone)
+	}
+	m.RegisterNewAdapterFunc(standaloneFileName, Download, newfunc)
+	m.RegisterNewAdapterFunc(standaloneFileName, Upload, newfunc)
+}
+
 // Initialise custom adapters based on current config
-func configureCustomAdapters(git Env, m *Manifest) {
+func configureCustomAdapters(git Env, m *concreteManifest) {
+	configureDefaultCustomAdapters(git, m)
+
 	pathRegex := regexp.MustCompile(`lfs.customtransfer.([^.]+).path`)
 	for k, _ := range git.All() {
 		match := pathRegex.FindStringSubmatch(k)
@@ -372,7 +386,7 @@ func configureCustomAdapters(git Env, m *Manifest) {
 		// Separate closure for each since we need to capture vars above
 		newfunc := func(name string, dir Direction) Adapter {
 			standalone := m.standaloneTransferAgent != ""
-			return newCustomAdapter(name, dir, path, args, concurrent, standalone)
+			return newCustomAdapter(m.fs, name, dir, path, args, concurrent, standalone)
 		}
 
 		if direction == "download" || direction == "both" {
