@@ -3,8 +3,8 @@ package tq
 import (
 	"net/http"
 
-	"github.com/git-lfs/git-lfs/lfsapi"
-	"github.com/git-lfs/git-lfs/tools"
+	"github.com/git-lfs/git-lfs/v3/lfsapi"
+	"github.com/git-lfs/git-lfs/v3/tools"
 	"github.com/rubyist/tracerx"
 )
 
@@ -35,10 +35,11 @@ func verifyUpload(c *lfsapi.Client, remote string, t *Transfer) error {
 		return err
 	}
 
+	req.Header.Set("Content-Type", "application/vnd.git-lfs+json")
+	req.Header.Set("Accept", "application/vnd.git-lfs+json")
 	for key, value := range action.Header {
 		req.Header.Set(key, value)
 	}
-	req.Header.Set("Content-Type", "application/vnd.git-lfs+json")
 
 	mv := c.GitEnv().Int(maxVerifiesConfigKey, defaultMaxVerifyAttempts)
 	mv = tools.MaxInt(defaultMaxVerifyAttempts, mv)
@@ -51,7 +52,7 @@ func verifyUpload(c *lfsapi.Client, remote string, t *Transfer) error {
 		if t.Authenticated {
 			res, err = c.Do(req)
 		} else {
-			res, err = c.DoWithAuth(remote, req)
+			res, err = c.DoWithAuth(remote, c.Endpoints.AccessFor(action.Href), req)
 		}
 
 		if err != nil {
