@@ -7,13 +7,14 @@ import (
 	"sync/atomic"
 	"testing"
 
-	"github.com/git-lfs/git-lfs/lfsapi"
+	"github.com/git-lfs/git-lfs/v3/lfsapi"
+	"github.com/git-lfs/git-lfs/v3/lfshttp"
 	"github.com/stretchr/testify/assert"
 	"github.com/stretchr/testify/require"
 )
 
 func TestVerifyWithoutAction(t *testing.T) {
-	c := &lfsapi.Client{}
+	c, _ := lfsapi.NewClient(nil)
 	tr := &Transfer{
 		Oid:  "abc",
 		Size: 123,
@@ -44,9 +45,14 @@ func TestVerifySuccess(t *testing.T) {
 	}))
 	defer srv.Close()
 
-	c, err := lfsapi.NewClient(nil, lfsapi.UniqTestEnv{
-		"lfs.transfer.maxverifies": "1",
-	})
+	// Set auth on the server URL but not on the /verify endpoint. Since auth
+	// will cause the request to fail, this will test that the correct access
+	// mode is being passed to `DoWithAuth()`
+	c, err := lfsapi.NewClient(lfshttp.NewContext(nil, nil, map[string]string{
+		"lfs.transfer.maxverifies":          "1",
+		"lfs." + srv.URL + ".access":        "Basic",
+		"lfs." + srv.URL + "/verify.access": "None",
+	}))
 	require.Nil(t, err)
 	tr := &Transfer{
 		Oid:  "abcd1234",
